@@ -1,7 +1,14 @@
 #ifdef E_TYPEDEFS
-#  ifndef HAVE_WAYLAND_ONLY
-#   include "e_comp_x.h"
-#  endif
+typedef struct _E_Comp_Wl_Aux_Hint  E_Comp_Wl_Aux_Hint;
+typedef struct _E_Comp_Wl_Buffer E_Comp_Wl_Buffer;
+typedef struct _E_Comp_Wl_Subsurf_Data E_Comp_Wl_Subsurf_Data;
+typedef struct _E_Comp_Wl_Surface_State E_Comp_Wl_Surface_State;
+typedef struct _E_Comp_Wl_Client_Data E_Comp_Wl_Client_Data;
+typedef struct _E_Comp_Wl_Output E_Comp_Wl_Output;
+typedef struct _E_Comp_Wl_Pointer E_Comp_Wl_Pointer;
+typedef struct E_Shell_Data E_Shell_Data;
+typedef struct Tizen_Extensions Tizen_Extensions;
+typedef void (*E_Comp_Wl_Grab_End_Cb)(E_Client*);
 #else
 # ifndef E_COMP_WL_H
 #  define E_COMP_WL_H
@@ -39,16 +46,18 @@
 #  define container_of(ptr, type, member) \
    ({ \
       const __typeof__( ((type *)0)->member ) *__mptr = (ptr); \
-      (type *)( (char *)__mptr - offsetof(type,member) ); \
+      (type *)(void *)( (char *)__mptr - offsetof(type,member) ); \
    })
 
-typedef struct _E_Comp_Wl_Buffer E_Comp_Wl_Buffer;
-typedef struct _E_Comp_Wl_Subsurf_Data E_Comp_Wl_Subsurf_Data;
-typedef struct _E_Comp_Wl_Surface_State E_Comp_Wl_Surface_State;
-typedef struct _E_Comp_Wl_Client_Data E_Comp_Wl_Client_Data;
-typedef struct _E_Comp_Wl_Data E_Comp_Wl_Data;
-typedef struct _E_Comp_Wl_Output E_Comp_Wl_Output;
-typedef struct E_Shell_Data E_Shell_Data;
+struct _E_Comp_Wl_Aux_Hint
+{
+   int           id;
+   const char   *hint;
+   const char   *val;
+   int32_t       fd;
+   Eina_Bool     changed;
+   Eina_Bool     deleted;
+};
 
 struct _E_Comp_Wl_Buffer
 {
@@ -58,9 +67,11 @@ struct _E_Comp_Wl_Buffer
    struct wl_listener deferred_destroy_listener;
    struct wl_shm_buffer *shm_buffer;
    struct wl_shm_pool *pool;
+   struct linux_dmabuf_buffer *dmabuf_buffer;
    E_Pixmap *discarding_pixmap;
    int32_t w, h;
    uint32_t busy;
+   Eina_Bool destroyed;
 };
 
 struct _E_Comp_Wl_Surface_State
@@ -92,6 +103,47 @@ struct _E_Comp_Wl_Subsurf_Data
    Eina_Bool synchronized;
 };
 
+typedef struct E_Comp_Wl_Extension_Data
+{
+   struct
+     {
+        struct wl_global *global;
+     } session_recovery;
+   struct
+     {
+        struct wl_global *global;
+     } www;
+   /* begin xdg-foreign */
+   struct
+     {
+        struct wl_global *global;
+        Eina_Hash *surfaces;
+     } zxdg_exporter_v1;
+   struct
+     {
+        struct wl_global *global;
+     } zxdg_importer_v1;
+   /* end xdg-foreign */
+   struct
+     {
+        struct wl_global *global;
+        Eina_List *resources;
+     } zwp_relative_pointer_manager_v1;
+   struct
+     {
+        struct wl_global *global;
+        Eina_Hash *constraints;
+     } zwp_pointer_constraints_v1;
+   struct
+   {
+     struct wl_global *global;
+   } action_route;
+   struct
+     {
+        struct wl_global *global;
+     } efl_aux_hints;
+} E_Comp_Wl_Extension_Data;
+
 struct _E_Comp_Wl_Data
 {
    Ecore_Wl2_Display *ewd;
@@ -100,6 +152,7 @@ struct _E_Comp_Wl_Data
      {
         struct wl_display *disp;
         Ecore_Wl2_Display *client_disp;
+        E_Client *client_ec;
         struct wl_registry *registry; // only used for nested wl compositors
         /* struct wl_event_loop *loop; */
         Eina_Inlist *globals;  // only used for nested wl compositors
@@ -135,22 +188,14 @@ struct _E_Comp_Wl_Data
         /*        } seat, output; */
      } signals;
 
-   struct
-     {
-        struct wl_resource *shell;
-        struct wl_resource *xdg_shell;
-     } shell_interface;
-
    struct
      {
         Eina_List *resources;
         Eina_List *focused;
         Eina_Bool enabled : 1;
-        xkb_mod_index_t mod_shift, mod_caps;
-        xkb_mod_index_t mod_ctrl, mod_alt;
-        xkb_mod_index_t mod_super;
         xkb_mod_mask_t mod_depressed, mod_latched, mod_locked;
         xkb_layout_index_t mod_group;
+        xkb_layout_index_t choosen_group;
         struct wl_array keys;
         struct wl_resource *focus;
         int mod_changed;
@@ -159,9 +204,9 @@ struct _E_Comp_Wl_Data
    struct
      {
         Eina_List *resources;
-        wl_fixed_t x, y;
-        wl_fixed_t grab_x, grab_y;
+        Evas_Coord x, y;
         uint32_t button;
+        uint32_t serial[2]; //down/up
         uint32_t button_mask;
         E_Client *ec;
         Eina_Bool enabled : 1;
@@ -207,7 +252,7 @@ struct _E_Comp_Wl_Data
      {
         void *source;
         struct wl_listener listener;
-        E_Client *xwl_owner;
+        Ecore_Window xwl_owner;
      } clipboard;
 
    struct
@@ -226,12 +271,7 @@ struct _E_Comp_Wl_Data
         char *area;
      } xkb;
 
-   struct
-     {
-        struct wl_global *global;
-        struct wl_client *client;
-        void (*read_pixels)(E_Comp_Wl_Output *output, void *pixels);
-     } screenshooter;
+   E_Comp_Wl_Extension_Data *extensions;
 
    Eina_List *outputs;
 
@@ -244,6 +284,9 @@ struct _E_Comp_Wl_Data
    E_Drag *drag;
    E_Client *drag_client;
    void *drag_source;
+
+   Eina_Bool dmabuf_disable : 1;
+   Eina_Bool dmabuf_proxy : 1;
 };
 
 struct _E_Comp_Wl_Client_Data
@@ -273,29 +316,65 @@ struct _E_Comp_Wl_Client_Data
         void (*unmap)(struct wl_resource *resource);
         Eina_Rectangle window;
         E_Shell_Data *data;
+        struct
+        {
+           Evas_Coord_Size min_size;
+           Evas_Coord_Size max_size;
+           Eina_Bool fullscreen : 1;
+           Eina_Bool unfullscreen : 1;
+           Eina_Bool maximize : 1;
+           Eina_Bool unmaximize : 1;
+           Eina_Bool minimize : 1;
+        } set;
      } shell;
+   struct
+   {
+      struct wl_resource *surface;
+      int x, y;
+   } www;
 
    E_Comp_Wl_Surface_State pending;
 
    Eina_List *frames;
+   Eina_List *constraints;
 
    struct
      {
         int32_t x, y;
      } popup;
+
+   Tizen_Extensions *tizen;
+   struct
+     {
+        Eina_Bool  changed : 1;
+        Eina_List *hints;
+        Eina_Bool  use_msg : 1;
+     } aux_hint;
+
+   int32_t on_outputs; /* Bitfield of the outputs this client is present on */
+
+   E_Maximize max;
+   E_Maximize unmax;
 #ifndef HAVE_WAYLAND_ONLY
    E_Pixmap *xwayland_pixmap;
    E_Comp_X_Client_Data *xwayland_data;
 #endif
 
-   Eina_Bool keep_buffer : 1;
+   int early_frame;
+
    Eina_Bool mapped : 1;
-   Eina_Bool change_icon : 1;
    Eina_Bool evas_init : 1;
-   Eina_Bool set_win_type : 1;
-   Eina_Bool frame_update : 1;
-   Eina_Bool maximize_pre : 1;
    Eina_Bool cursor : 1;
+   Eina_Bool moved : 1;
+   Eina_Bool maximizing : 1;
+   Eina_Bool in_commit : 1;
+   Eina_Bool is_xdg_surface : 1;
+   Eina_Bool grab : 1;
+   Eina_Bool buffer_commit : 1;
+   Eina_Bool need_xdg6_configure : 1;
+   Eina_Bool maximize_anims_disabled : 1;
+   Eina_Bool ssd_mouse_in : 1;
+   Eina_Bool need_center : 1;
 };
 
 struct _E_Comp_Wl_Output
@@ -316,6 +395,14 @@ struct _E_Comp_Wl_Output
    void *data;
 };
 
+struct _E_Comp_Wl_Pointer
+{
+   E_Client *cursor;
+   Evas_Point offset;
+   Eina_Bool entered : 1;
+   Eina_Bool cursor_set : 1;
+};
+
 E_API Eina_Bool e_comp_wl_init(void);
 EINTERN void e_comp_wl_shutdown(void);
 
@@ -327,15 +414,35 @@ E_API E_Comp_Wl_Buffer *e_comp_wl_buffer_get(struct wl_resource *resource);
 
 E_API struct wl_signal e_comp_wl_surface_create_signal_get(void);
 E_API double e_comp_wl_idle_time_get(void);
-E_API Eina_Bool e_comp_wl_output_init(const char *id, const char *make, const char *model, int x, int y, int w, int h, int pw, int ph, unsigned int refresh, unsigned int subpixel, unsigned int transform);
+E_API Eina_Bool e_comp_wl_output_init(const char *id, const char *make, const char *model, int x, int y, int w, int h, int pw, int ph, unsigned int refresh, unsigned int subpixel, unsigned int transform, unsigned int num);
 E_API void e_comp_wl_output_remove(const char *id);
 
-EINTERN Eina_Bool e_comp_wl_key_down(Ecore_Event_Key *ev);
-EINTERN Eina_Bool e_comp_wl_key_up(Ecore_Event_Key *ev);
+EINTERN Eina_Bool e_comp_wl_key_down(Ecore_Event_Key *ev, E_Client *ec);
+EINTERN Eina_Bool e_comp_wl_key_up(Ecore_Event_Key *ev, E_Client *ec);
 E_API Eina_Bool e_comp_wl_evas_handle_mouse_button(E_Client *ec, uint32_t timestamp, uint32_t button_id, uint32_t state);
 
 E_API extern int E_EVENT_WAYLAND_GLOBAL_ADD;
 
+E_API void e_comp_wl_grab_client_add(E_Client *ec, E_Comp_Wl_Grab_End_Cb cb);
+E_API void e_comp_wl_grab_client_del(E_Client *ec, Eina_Bool dismiss);
+E_API Eina_Bool e_comp_wl_client_is_grabbed(const E_Client *ec);
+E_API Eina_Bool e_comp_wl_grab_client_mouse_move(const Ecore_Event_Mouse_Move *ev);
+E_API Eina_Bool e_comp_wl_grab_client_mouse_button(const Ecore_Event_Mouse_Button *ev);
+
+E_API void e_comp_wl_extension_relative_motion_event(uint64_t time_usec, double dx, double dy, double dx_unaccel, double dy_unaccel);
+E_API void e_comp_wl_extension_pointer_constraints_commit(E_Client *ec);
+E_API Eina_Bool e_comp_wl_extension_pointer_constraints_update(E_Client *ec, int x, int y);
+E_API void e_comp_wl_extension_pointer_unconstrain(E_Client *ec);
+E_API void e_comp_wl_extension_action_route_pid_allowed_set(uint32_t pid, Eina_Bool allow);
+E_API const void *e_comp_wl_extension_action_route_interface_get(int *version);
+
+
+E_API void
+e_policy_wl_aux_message_send(E_Client *ec,
+                             const char *key,
+                             const char *val,
+                             Eina_List *options);
+
 # ifndef HAVE_WAYLAND_ONLY
 EINTERN void e_comp_wl_xwayland_client_queue(E_Client *ec);
 static inline E_Comp_X_Client_Data *
